//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      Calculator.hpp
//

#ifndef __SOPHIACRADLE_CALCULATOR_HPP
#define __SOPHIACRADLE_CALCULATOR_HPP

#include <SophiaFramework.hpp>
#include "KeySequence.hpp"


#define CONFIG_FILENAME "config.dat"    // Config file


SFMTYPEDEFCLASS(Monitor)

SFMTYPEDEFCLASS(Calculator)

class Calculator {

    SFMSEALCOPY(Calculator)

    public:

        enum {
            ERR_SYNTAX = 0x9000,            // Syntax error
            ERR_ZERO_DIVISION = 0x9001,     // 0 Division
            ERR_DOMAIN = 0x9002,            // Region definition error
            ERR_PLUS_INF = 0x9003,          // + Infinity
            ERR_MINUS_INF = 0x9004,         // - Infinity
            ERR_NAN = 0x9005                // NaN
        };
        enum NumberTypeEnum {
            NUMBERTYPE_BIN = 0,
            NUMBERTYPE_OCT,
            NUMBERTYPE_DEC,
            NUMBERTYPE_HEX,
            NUMBERTYPE_LIMIT
        };
        enum SignTypeEnum {
            SIGNTYPE_S = 0,
            SIGNTYPE_U,
            SIGNTYPE_LIMIT
        };
        enum BitTypeEnum {
            BITTYPE_08 = 0,
            BITTYPE_16,
            BITTYPE_32,
            BITTYPE_64,
            BITTYPE_LIMIT
        };
        enum AngleTypeEnum {
            ANGLETYPE_DEG = 0,
            ANGLETYPE_RAD,
            ANGLETYPE_GRA,
            ANGLETYPE_LIMIT
        };
        enum ConfigTagEnum {                // Config file tag number
            CONFIGTAG_ANSINT = 0x0000,      // ans of Integer version
            CONFIGTAG_ANSREAL = 0x0001,     // ans of Real number version
            CONFIGTAG_ANSOCT = 0x0002,      // ans is an integer or a real number
            CONFIGTAG_TYPENUMBER = 0x0003,
            CONFIGTAG_TYPESIGN = 0x0004,
            CONFIGTAG_TYPEBIT = 0x0005,
            CONFIGTAG_TYPEANGLE = 0x0006,
            CONFIGTAG_LIMIT
        };
        enum UnitEnum {                     // Unit
            UNIT_FEMTO = 0,
            UNIT_PICO,
            UNIT_NANO,
            UNIT_MICRO,
            UNIT_MILLI,
            UNIT_NONE,
            UNIT_KILO,
            UNIT_MEGA,
            UNIT_GIGA,
            UNIT_TERA,
            UNIT_LIMIT
        };
        enum AnsOctEnum {
            ANSOCT_OCT = 0,
            ANSOCT_REAL,
            ANSOCT_LIMIT
        };

    private:
        MonitorPtr _monitor;
        NumberTypeEnum _number;
        SignTypeEnum _sign;
        BitTypeEnum _bit;
        AngleTypeEnum _angle;
        KeySequence _sequence;
        UInt64 _lastAnsInt;
        Float64 _lastAnsReal;
        AnsOctEnum _ansOct;
        NumberTypeEnum _lastNumber;
        SignTypeEnum _lastSign;
        BitTypeEnum _lastBit;
        AngleTypeEnum _lastAngle;
        SInt16 _unit;                       // Present unit of display

    public:

        explicit Calculator(MonitorPtr monitor);
        ~Calculator(Void);
        Void SetNumberType(NumberTypeEnum param);
        NumberTypeEnum GetNumberType(Void) const;
        Void SetSignType(SignTypeEnum param);
        SignTypeEnum GetSignType(Void) const;
        Void SetBitType(BitTypeEnum param);
        BitTypeEnum GetBitType(Void) const;
        Void SetAngleType(AngleTypeEnum param);
        AngleTypeEnum GetAngleType(Void) const;
        SFCError Initialize(Void);
        SFCError Terminate(Void);
        Void DisplayReal(Float64 val);
        Void New(Void);
        Void Dispose(Void);
        Void Continue(Void);
        SFCError Execute(Void);
        Void View_left(Void);
        Void View_right(Void);
        Void View_plus(Void);
        Void View_minus(Void);
        Void Edit_clear(Void);
        Void Edit_up(Void);
        Void Edit_down(Void);
        Void Edit_left(Void);
        Void Edit_right(Void);
        Void Number_0(Void);
        Void Number_1(Void);
        Void Number_2(Void);
        Void Number_3(Void);
        Void Number_4(Void);
        Void Number_5(Void);
        Void Number_6(Void);
        Void Number_7(Void);
        Void Number_8(Void);
        Void Number_9(Void);
        Void Number_A(Void);
        Void Number_B(Void);
        Void Number_C(Void);
        Void Number_D(Void);
        Void Number_E(Void);
        Void Number_F(Void);
        Void Number_period(Void);
        Void Number_pi(Void);
        Void Number_e(Void);
        Void Number_Ran(Void);
        Void Number_Ans(Void);
        Void Parenthesis_left(Void);
        Void Parenthesis_right(Void);
        Void Unit_T(Void);
        Void Unit_G(Void);
        Void Unit_M(Void);
        Void Unit_K(Void);
        Void Unit_m(Void);
        Void Unit_u(Void);
        Void Unit_n(Void);
        Void Unit_p(Void);
        Void Unit_f(Void);
        Void Unit_EXP(Void);
        Void Operator_add(Void);
        Void Operator_sub(Void);
        Void Operator_mul(Void);
        Void Operator_div(Void);
        Void Operator_mod(Void);
        Void Operator_not(Void);
        Void Operator_and(Void);
        Void Operator_ior(Void);
        Void Operator_xor(Void);
        Void Operator_comma(Void);
        Void Function_int(Void);
        Void Function_fix(Void);
        Void Function_sqrt(Void);
        Void Function_xy(Void);
        Void Function_log(Void);
        Void Function_ln(Void);
        Void Function_fact(Void);
        Void Function_nCr(Void);
        Void Function_jn(Void);
        Void Function_abs(Void);
        Void Function_frac(Void);
        Void Function_cbrt(Void);
        Void Function_pow(Void);
        Void Function_exp(Void);
        Void Function_gamma(Void);
        Void Function_nPr(Void);
        Void Function_yn(Void);
        Void Function_sin(Void);
        Void Function_cos(Void);
        Void Function_tan(Void);
        Void Function_sinh(Void);
        Void Function_cosh(Void);
        Void Function_tanh(Void);
        Void Function_asin(Void);
        Void Function_acos(Void);
        Void Function_atan(Void);
        Void Function_asinh(Void);
        Void Function_acosh(Void);
        Void Function_atanh(Void);

    private:

        static SFXAnsiString UInt64ToAnswer(UInt64 value, Calculator::SignTypeEnum sign, Calculator::BitTypeEnum bit, Calculator::NumberTypeEnum number);
        static Void CheckDigit(UInt08Ptr pbuf, SInt16Ptr porder, SInt16Ptr plast);
        static Void Round(UInt08Ptr pbuf, SInt16Ptr porder, SInt16Ptr plast, UInt16 digit);
        static Void DisplayRealInner(UInt08Ptr pbuf, SInt16Ptr porder, SInt16Ptr plast, Float64 val);
        Void SetKey(KeyType::KeyCode code);
        SFCError CalculateSInt08(Void);
        SFCError CalculateUInt08(Void);
        SFCError CalculateSInt16(Void);
        SFCError CalculateUInt16(Void);
        SFCError CalculateSInt32(Void);
        SFCError CalculateUInt32(Void);
        SFCError CalculateSInt64(Void);
        SFCError CalculateUInt64(Void);
        SFCError CalculateReal(Void);
};

inline Calculator::NumberTypeEnum Calculator::GetNumberType(Void) const
{
    return _number;
}

inline Calculator::SignTypeEnum Calculator::GetSignType(Void) const
{
    return _sign;
}

inline Calculator::BitTypeEnum Calculator::GetBitType(Void) const
{
    return _bit;
}

inline Calculator::AngleTypeEnum Calculator::GetAngleType(Void) const
{
    return _angle;
}

inline Void Calculator::Number_0(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_0);
    return;
}

inline Void Calculator::Number_1(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_1);
    return;
}

inline Void Calculator::Number_2(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_2);
    return;
}

inline Void Calculator::Number_3(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_3);
    return;
}

inline Void Calculator::Number_4(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_4);
    return;
}

inline Void Calculator::Number_5(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_5);
    return;
}

inline Void Calculator::Number_6(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_6);
    return;
}

inline Void Calculator::Number_7(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_7);
    return;
}

inline Void Calculator::Number_8(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_8);
    return;
}

inline Void Calculator::Number_9(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_9);
    return;
}

inline Void Calculator::Number_A(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_A);
    return;
}

inline Void Calculator::Number_B(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_B);
    return;
}

inline Void Calculator::Number_C(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_C);
    return;
}

inline Void Calculator::Number_D(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_D);
    return;
}

inline Void Calculator::Number_E(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_E);
    return;
}

inline Void Calculator::Number_F(Void)
{
    SetKey(KeyType::KEYCODE_NUMBER_F);
    return;
}

inline Void Calculator::Number_period(Void)
{
    SetKey(KeyType::KEYCODE_FLOAT_POINT);
    return;
}

inline Void Calculator::Number_pi(Void)
{
    SetKey(KeyType::KEYCODE_CONSTANT_PI);
    return;
}

inline Void Calculator::Number_e(Void)
{
    SetKey(KeyType::KEYCODE_CONSTANT_E);
    return;
}

inline Void Calculator::Number_Ran(Void)
{
    SetKey(KeyType::KEYCODE_CONSTANT_RAN);
    return;
}

inline Void Calculator::Number_Ans(Void)
{
    SetKey(KeyType::KEYCODE_MEMORY_ANS);
    return;
}

inline Void Calculator::Parenthesis_left(Void)
{
    SetKey(KeyType::KEYCODE_PARENTHESIS_LEFT);
    return;
}

inline Void Calculator::Parenthesis_right(Void)
{
    SetKey(KeyType::KEYCODE_PARENTHESIS_RIGHT);
    return;
}

inline Void Calculator::Unit_T(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_TERA);
    return;
}

inline Void Calculator::Unit_G(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_GIGA);
    return;
}

inline Void Calculator::Unit_M(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_MEGA);
    return;
}

inline Void Calculator::Unit_K(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_KILO);
    return;
}

inline Void Calculator::Unit_m(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_MILLI);
    return;
}

inline Void Calculator::Unit_u(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_MICRO);
    return;
}

inline Void Calculator::Unit_n(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_NANO);
    return;
}

inline Void Calculator::Unit_p(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_PICO);
    return;
}

inline Void Calculator::Unit_f(Void)
{
    SetKey(KeyType::KEYCODE_UNIT_FEMTO);
    return;
}

inline Void Calculator::Unit_EXP(Void)
{
    SetKey(KeyType::KEYCODE_FLOAT_EXP);
    return;
}

inline Void Calculator::Operator_add(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_PLUS);
    return;
}

inline Void Calculator::Operator_sub(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_MINUS);
    return;
}

inline Void Calculator::Operator_mul(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_MUL);
    return;
}

inline Void Calculator::Operator_div(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_DIV);
    return;
}

inline Void Calculator::Operator_mod(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_MOD);
    return;
}

inline Void Calculator::Operator_not(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_NOT);
    return;
}

inline Void Calculator::Operator_and(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_AND);
    return;
}

inline Void Calculator::Operator_ior(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_OR);
    return;
}

inline Void Calculator::Operator_xor(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_XOR);
    return;
}

inline Void Calculator::Operator_comma(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_SEPARATOR);
    return;
}

inline Void Calculator::Function_int(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_INT);
    return;
}

inline Void Calculator::Function_fix(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_FIX);
    return;
}

inline Void Calculator::Function_sqrt(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_SQRT);
    return;
}

inline Void Calculator::Function_xy(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_POW);
    return;
}

inline Void Calculator::Function_log(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_LOG);
    return;
}

inline Void Calculator::Function_ln(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_LN);
    return;
}

inline Void Calculator::Function_fact(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_FACTORIAL);
    return;
}

inline Void Calculator::Function_nCr(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_NCR);
    return;
}

inline Void Calculator::Function_jn(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_JN);
    return;
}

inline Void Calculator::Function_abs(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_ABS);
    return;
}

inline Void Calculator::Function_frac(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_FRAC);
    return;
}

inline Void Calculator::Function_cbrt(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_CBRT);
    return;
}

inline Void Calculator::Function_pow(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_POW);
    return;
}

inline Void Calculator::Function_exp(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_EXP);
    return;
}

inline Void Calculator::Function_gamma(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_GAMMA);
    return;
}

inline Void Calculator::Function_nPr(Void)
{
    SetKey(KeyType::KEYCODE_OPERATOR_NPR);
    return;
}

inline Void Calculator::Function_yn(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_YN);
    return;
}

inline Void Calculator::Function_sin(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_SIN);
    return;
}

inline Void Calculator::Function_cos(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_COS);
    return;
}

inline Void Calculator::Function_tan(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_TAN);
    return;
}

inline Void Calculator::Function_sinh(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_SINH);
    return;
}

inline Void Calculator::Function_cosh(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_COSH);
    return;
}

inline Void Calculator::Function_tanh(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_TANH);
    return;
}

inline Void Calculator::Function_asin(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_ASIN);
    return;
}

inline Void Calculator::Function_acos(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_ACOS);
    return;
}

inline Void Calculator::Function_atan(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_ATAN);
    return;
}

inline Void Calculator::Function_asinh(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_ASINH);
    return;
}

inline Void Calculator::Function_acosh(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_ACOSH);
    return;
}

inline Void Calculator::Function_atanh(Void)
{
    SetKey(KeyType::KEYCODE_FUNCTION_ATANH);
    return;
}

#endif // __SOPHIACRADLE_CALCULATOR_HPP //
