//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//     KeySequence.hpp
//

#ifndef __SOPHIACRADLE_KEYSEQUENCE_HPP
#define __SOPHIACRADLE_KEYSEQUENCE_HPP

#include <SophiaFramework.hpp>
#include "Array.hpp"
#include "Cursor.hpp"

SFMTYPEDEFCLASS(KeyType)

class KeyType {
//  Class that handles key information

public:

    enum KeyCode {
        // Add it at the end of each classification (the order is important)
        KEYCODE_NONE = 0x0000,              // Undefined
        KEYCODE_NUMBER_0 = 0x0100,          // 0 Key
        KEYCODE_NUMBER_1 = 0x0101,          // 1 Key
        KEYCODE_NUMBER_2 = 0x0102,          // 2 Key
        KEYCODE_NUMBER_3 = 0x0103,          // 3 Key
        KEYCODE_NUMBER_4 = 0x0104,          // 4 Key
        KEYCODE_NUMBER_5 = 0x0105,          // 5 Key
        KEYCODE_NUMBER_6 = 0x0106,          // 6 Key
        KEYCODE_NUMBER_7 = 0x0107,          // 7 Key
        KEYCODE_NUMBER_8 = 0x0108,          // 8 Key
        KEYCODE_NUMBER_9 = 0x0109,          // 9 Key
        KEYCODE_NUMBER_A = 0x010A,          // A Key
        KEYCODE_NUMBER_B = 0x010B,          // B Key
        KEYCODE_NUMBER_C = 0x010C,          // C Key
        KEYCODE_NUMBER_D = 0x010D,          // D Key
        KEYCODE_NUMBER_E = 0x010E,          // E Key
        KEYCODE_NUMBER_F = 0x010F,          // F Key
        KEYCODE_CONSTANT_PI = 0x0200,       // PI Key
        KEYCODE_CONSTANT_E = 0x0201,        // Number of Napier Key
        KEYCODE_CONSTANT_RAN = 0x0202,      // Random number (although it is not a constant, classify it here by relation of processing)
        KEYCODE_UNIT_TERA = 0x0400,         // Tera Key
        KEYCODE_UNIT_GIGA = 0x0401,         // GiGa Key
        KEYCODE_UNIT_MEGA = 0x0402,         // Mega Key
        KEYCODE_UNIT_KILO = 0x0403,         // Kilo Key
        KEYCODE_UNIT_MILLI = 0x0404,        // Milli Key
        KEYCODE_UNIT_MICRO = 0x0405,        // Micro Key
        KEYCODE_UNIT_NANO = 0x0406,         // Nano Key
        KEYCODE_UNIT_PICO = 0x0407,         // Pico Key
        KEYCODE_UNIT_FEMTO = 0x0408,        // Femto Key
        KEYCODE_FLOAT_POINT = 0x0800,       // . Key
        KEYCODE_FLOAT_EXP = 0x0801,         // EXP Key
        KEYCODE_PARENTHESIS_LEFT = 0x1000,  // ( Key
        KEYCODE_PARENTHESIS_RIGHT = 0x1001, // ) Key
        KEYCODE_MEMORY_ANS = 0x2000,        // ANS Key
        KEYCODE_OPERATOR_PLUS = 0x4000,     // + Key
        KEYCODE_OPERATOR_MINUS = 0x4001,    // - Key
        KEYCODE_OPERATOR_MUL = 0x4002,      // * Key
        KEYCODE_OPERATOR_DIV = 0x4003,      // / Key
        KEYCODE_OPERATOR_POW = 0x4004,      // ^ Key
        KEYCODE_OPERATOR_FACTORIAL = 0x4005,// ! Key
        KEYCODE_OPERATOR_MOD = 0x4006,      // mod Key
        KEYCODE_OPERATOR_AND = 0x4007,      // and Key
        KEYCODE_OPERATOR_OR = 0x4008,       // or Key
        KEYCODE_OPERATOR_XOR = 0x4009,      // xor Key
        KEYCODE_OPERATOR_NOT = 0x400A,      // not Key
        KEYCODE_OPERATOR_NPR = 0x400B,      // nPr Key
        KEYCODE_OPERATOR_NCR = 0x400C,      // nCr Key
        KEYCODE_OPERATOR_SMUL = 0x400D,     // the strongest * (For inside)
        KEYCODE_FUNCTION_SEPARATOR = 0x8000,    // , Key
        KEYCODE_FUNCTION_POW = 0x8001,      // pow Method Key
        KEYCODE_FUNCTION_EXP = 0x8002,      // exp Method Key
        KEYCODE_FUNCTION_LN = 0x8003,       // ln Method Key
        KEYCODE_FUNCTION_LOG = 0x8004,      // log Method Key
        KEYCODE_FUNCTION_SQRT = 0x8005,     // sqrt Method Key
        KEYCODE_FUNCTION_CBRT = 0x8006,     // cbrt Method Key
        KEYCODE_FUNCTION_SIN = 0x8007,      // sin Method Key
        KEYCODE_FUNCTION_COS = 0x8008,      // cos Method Key
        KEYCODE_FUNCTION_TAN = 0x8009,      // tan Method Key
        KEYCODE_FUNCTION_ASIN = 0x800A,     // asin Method Key
        KEYCODE_FUNCTION_ACOS = 0x800B,     // acos Method Key
        KEYCODE_FUNCTION_ATAN = 0x800C,     // atan Method Key
        KEYCODE_FUNCTION_SINH = 0x800D,     // sinh Method Key
        KEYCODE_FUNCTION_COSH = 0x800E,     // cosh Method Key
        KEYCODE_FUNCTION_TANH = 0x800F,     // tanh Method Key
        KEYCODE_FUNCTION_ASINH = 0x8010,    // asinh Method Key
        KEYCODE_FUNCTION_ACOSH = 0x8011,    // acosh Method Key
        KEYCODE_FUNCTION_ATANH = 0x8012,    // atanh Method Key
        KEYCODE_FUNCTION_ABS = 0x8013,      // abs Method Key
        KEYCODE_FUNCTION_INT = 0x8014,      // int Method Key
        KEYCODE_FUNCTION_FIX = 0x8015,      // fix Method Key
        KEYCODE_FUNCTION_GAMMA = 0x8016,    // Gamma Method Key
        KEYCODE_FUNCTION_FRAC = 0x8017,     // frac Method Key
        KEYCODE_FUNCTION_JN = 0x8018,       // jn Key
        KEYCODE_FUNCTION_YN = 0x8019        // yn Key
    };
    enum KeyClass {
        KEYCLASS_NONE = 0x0000,         // No classification
        KEYCLASS_NUMBER = 0x0100,       // Number Key
        KEYCLASS_CONSTANT = 0x0200,     // Constant Key
        KEYCLASS_UNIT = 0x0400,         // Unit Key
        KEYCLASS_FLOAT = 0x0800,        // For real number Key
        KEYCLASS_PARENTHESIS = 0x1000,  // Parenthesis Key
        KEYCLASS_MEMORY = 0x2000,       // Memory Key
        KEYCLASS_OPERATOR = 0x4000,     // Operator Key
        KEYCLASS_FUNCTION = 0x8000      // Method Key
    };

private:
    KeyCode _keyCode;       // Key Code

public:
    explicit KeyType(Void);
    explicit KeyType(KeyCode code);
    KeyType(KeyTypeConstRef ob);
    KeyTypeConstRef operator=(KeyTypeConstRef ob);
    Void SetKeyCode(KeyCode code);
    KeyCode GetKeyCode(Void) const;
    SFXAnsiString GetString(Void) const;
    SInt16 GetLength(Void) const;
};

inline KeyType::KeyType(Void) : _keyCode(KeyType::KEYCODE_NONE)
{
}

inline KeyType::KeyType(KeyType::KeyCode code) : _keyCode(code)
{
}

inline KeyType::KeyType(KeyTypeConstRef ob) : _keyCode(ob._keyCode)
{
}

inline KeyTypeConstRef KeyType::operator=(KeyTypeConstRef ob)
{
    _keyCode = ob._keyCode;
    return *this;
}

inline Void KeyType::SetKeyCode(KeyType::KeyCode code)
{
    _keyCode = code;
}

inline KeyType::KeyCode KeyType::GetKeyCode(Void) const
{
    return _keyCode;
}

inline SInt16 KeyType::GetLength(Void) const
{
    SFXAnsiString str(GetString());
    return static_cast<SInt16>(str.GetLength());
}


SFMTYPEDEFCLASS(KeySequence)
class KeySequence {
private:
    Array<KeyType> _keys;

public:
    explicit KeySequence(Void);
    KeySequence(KeySequenceConstRef ob);
    KeySequenceConstRef operator=(KeySequenceConstRef ob);
    SFCError Set(KeySequenceConstRef ob);
    SFCError Append(KeyTypeConstRef key);
    Void Remove(SInt32 index);
    Void Clear(Void);
    SFCError Insert(SInt32 index, KeyTypeConstRef key);
    SInt32 GetSize(Void) const;
    KeyTypeConstRef GetKeyType(SInt32 index) const;
    SFCError Normalize(Void);
    SFXAnsiString GetString(Void) const;
    Cursor GetCorrectCursor(SInt16 position) const;
    Cursor GetCorrectCursor(CursorConstRef cursor) const;
    Cursor GetPreviousCursor(SInt16 position) const;
    Cursor GetPreviousCursor(CursorConstRef cursor) const;
    Cursor GetNextCursor(SInt16 position) const;
    Cursor GetNextCursor(CursorConstRef cursor) const;
    SInt32 GetIndex(SInt16 position) const;
    SInt32 GetIndex(CursorConstRef cursor) const;
    Cursor GetCursor(SInt32 index) const;
};

inline KeySequence::KeySequence(Void)
{
}

inline KeySequence::KeySequence(KeySequenceConstRef ob) : _keys(ob._keys)
{
}

inline KeySequenceConstRef KeySequence::operator=(KeySequenceConstRef ob)
{
    _keys = ob._keys;
    return *this;
}

inline SFCError KeySequence::Set(KeySequenceConstRef ob)
{
    return _keys.Set(ob._keys);
}

inline SFCError KeySequence::Append(KeyTypeConstRef key)
{
    return _keys.Append(key);
}

inline Void KeySequence::Remove(SInt32 index)
{
    _keys.Remove(index);
}

inline Void KeySequence::Clear(Void)
{
    _keys.Clear();
}

inline SFCError KeySequence::Insert(SInt32 index, KeyTypeConstRef key)
{
    return _keys.Insert(index, key);
}

inline SInt32 KeySequence::GetSize(Void) const
{
    return _keys.GetSize();
}

inline KeyTypeConstRef KeySequence::GetKeyType(SInt32 index) const
{
    return _keys.Get(index);
}

inline Cursor KeySequence::GetCorrectCursor(CursorConstRef cursor) const
{
    return GetCorrectCursor(cursor.GetPosition());
}

inline Cursor KeySequence::GetPreviousCursor(CursorConstRef cursor) const
{
    return GetPreviousCursor(cursor.GetPosition());
}

inline Cursor KeySequence::GetNextCursor(CursorConstRef cursor) const
{
    return GetNextCursor(cursor.GetPosition());
}

inline SInt32 KeySequence::GetIndex(CursorConstRef cursor) const
{
    return GetIndex(cursor.GetPosition());
}

#endif // __SOPHIACRADLE_KEYSEQUENCE_HPP //
