//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      Keyboard.cpp
//

#include "Keyboard.hpp"
#include "psec_res.h"
#include "psec.hpp"
#include "Softkey.hpp"
#include "MNKeyboard.hpp"
#include "VNKeyboard.hpp"
#include "PDialog.hpp"
#include "VDialog.hpp"

#define     LABEL_SOFTKEY1          ("Clear")
#define     LABEL_SOFTKEY2          ("Set")
#define     LABEL_SOFTKEY3          ("=")
#define     MESSAGEDIALOG_HEIGHT    (104)
#define     MULTIDIALOG_HEIGHT      (136)

enum {
    OPERATION_PREFERENCE,
    OPERATION_VERSION,
    OPERATION_LIMIT
};

Keyboard::Keyboard(SFRApplicationPtr director, SFXRectangleConstRef rect, SFCType attribute, MonitorPtr monitor, CalculatorPtr calculator) static_throws : SFRPlainWindow(director, rect, BEHAVIOR_SFRPLAINWINDOW & ~STATUS_VISIBLE, attribute), _monitor(monitor), _calculator(calculator)
{
    static KeyRecConst key = {{
        {AVK_UNDEFINED, AVK_CLR, AVK_UNDEFINED},
        {        AVK_1,   AVK_2,         AVK_3},
        {        AVK_4,   AVK_5,         AVK_6},
        {        AVK_7,   AVK_8,         AVK_9},
        {     AVK_STAR,   AVK_0,     AVK_POUND}
    }};
    SoftkeyPtr softkey;
    SFRButtonControlPtr button;
    SFXRectangle cell;
    SFXRectangle temp;
    SInt16 i;
    SInt16 j;

    Softkey::Initialize(this);
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnRender)));
        if (static_try()) {
            static_throw(RegisterHandler(static_cast<SFCEventEnum>(Softkey::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnSoftkey)));
            if (static_try()) {
#if TARGET_VERSION_GT(3, 1, 0)
                static_throw(RegisterHandler(SFEVT_KEY, SFEVT_KEY_RELEASE, HANDLER_BEFORE, HANDLER_FUNCTION(OnKey)));
#else
                static_throw(RegisterHandler(SFEVT_KEY, SFEVT_KEY_HELD, HANDLER_BEFORE, HANDLER_FUNCTION(OnKey)));
#endif
                if (static_try()) {
                    cell = GetContentWorld();
                    cell.Deflate(0, KEY_MARGIN);
                    cell.DivSize(KEYCOUNT_HRIZ, KEYCOUNT_VERT);
                    for (j = 0; j < KEYCOUNT_VERT && static_try(); ++j) {
                        temp = cell;
                        temp.Deflate(1 + KEY_MARGIN, 1);
                        for (i = 0; i < KEYCOUNT_HRIZ && static_try(); ++i) {
                            if ((button = ::new SFRButtonControl(this, temp, "", BEHAVIOR_SFRBUTTONCONTROL & ~STATUS_FOCUS, key.event[j][i])) != null) {
                                static_throw(*button);
                                if (static_try()) {
                                    static_throw(button->RegisterHandler(SREVT_CONTROL, HANDLER_AFTER, HANDLER_FUNCTION(OnButton)));
                                }
                                else {
                                    ::delete button;
                                }
                            }
                            else {
                                static_throw(SFERR_NO_MEMORY);
                            }
                            temp.AddX(cell.GetWidth());
                        }
                        cell.AddY(cell.GetHeight());
                    }
                    if (static_try()) {
                        if ((softkey = Softkey::GetInstance()) != null) {
                            static_throw(softkey->Register(this, Softkey::SOFTKEY_1, LABEL_SOFTKEY1));
                            if (static_try()) {
                                static_throw(softkey->Register(this, Softkey::SOFTKEY_2, LABEL_SOFTKEY2));
                                if (static_try()) {
                                    static_throw(softkey->Register(this, Softkey::SOFTKEY_3, LABEL_SOFTKEY3));
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return;
}

Keyboard::~Keyboard(Void)
{
    Softkey::Terminate(this);
    return;
}

KeyboardPtr Keyboard::GetInstance(SFCType attribute)
{
    return static_cast<KeyboardPtr>(SFRApplication::GetInstance()->GetFront(TYPE_SFRWINDOW, attribute, BEHAVIOR_NONE));
}

Void Keyboard::Refresh(MapRecConstRef map)
{
    static SFXBevelColor::AtomRecConst theme = {
        {0x00, 0xFF, 0xFF, 0xFF},
        {0x00, 0xDD, 0xDD, 0xDD},
        {0x00, 0x88, 0x88, 0x88}
    };
    SFRButtonControlPtr button;
    SFXBevelColor temp;
    SInt16 i;
    SInt16 j;

    button = static_cast<SFRButtonControlPtr>(GetFront(TYPE_SFRCONTROL, ATTRIBUTE_WILDCARD, BEHAVIOR_NONE));
    for (j = 0; j < KEYCOUNT_VERT && button != null; ++j) {
        for (i = 0; i < KEYCOUNT_HRIZ && button != null; ++i) {
            button->SetTitle(map.state[j][i].title);
            temp = theme;
            temp.Sub(map.state[j][i].color);
            button->SetColor(SFRButtonControl::COLOR_LIGHT, temp.GetLight());
            button->SetColor(SFRButtonControl::COLOR_BASE, temp.GetBase());
            button->SetColor(SFRButtonControl::COLOR_SHADOW, temp.GetDark());
            button->SetStatusVisible(!button->GetTitle().IsEmpty());
            button = static_cast<SFRButtonControlPtr>(button->GetNext(TYPE_SFRCONTROL, ATTRIBUTE_WILDCARD, BEHAVIOR_NONE, false));
        }
    }
    InvalidateContent();
    return;
}

Void Keyboard::Activate(KeyboardPtr inactivate, UInt32 mode)
{
    SoftkeyPtr softkey;

    if (inactivate != null) {
        inactivate->SetStatusVisible(false);
    }
    SetStatusVisible(true);
    if ((softkey = Softkey::GetInstance()) != null) {
        softkey->Bind(this);
    }
    _monitor->SetMode(mode);
    return;
}

Void Keyboard::HandlerSoftkey1(Void)
{
    SFRMultiDialogPtr dialog;
    SFRMultiDialog::ParamRec param;

    param.text = "Is the expression deleted?";
    param.ok = "YES";
    param.cancel = "Cancel";
    param.focus = SFRMultiDialog::FOCUS_CANCEL;
    param.image = SFBShell::GetInstance()->LoadResImage(PSEC_RES_FILE, ACCEPT);
    if ((dialog = ::new SFRMultiDialog(psec::CalculateDialogPosition(SFRMultiDialog::GetMargin(), MULTIDIALOG_HEIGHT), "Confirmation", param)) != null) {
        psec::OpenStandardDialog(dialog, HANDLER_FUNCTION(FromSoftkey1));
    }
    return;
}

Void Keyboard::HandlerSoftkey2(Void)
{
    SFRTextMenuPtr menu;
    SFCError error(SFERR_NO_ERROR);

    if ((menu = ::new SFRTextMenu(psec::CalculateMenuPosition(OPERATION_LIMIT), "Menu", null, 0)) != null) {
        if ((error = menu->static_catch()) == SFERR_NO_ERROR) {
            if ((error = menu->AppendItem("Environment setting", AVK_1, 0xFBF6)) == SFERR_NO_ERROR) {
                if ((error = menu->AppendItem("Version", AVK_UNDEFINED)) == SFERR_NO_ERROR) {
                    error = psec::OpenStandardMenu(menu, HANDLER_FUNCTION(FromSoftkey2));
                }
            }
        }
        if (error != SFERR_NO_ERROR) {
            ::delete menu;
        }
    }
    return;
}

Void Keyboard::HandlerSelect(Void)
{
    SFRMessageDialogPtr dialog;
    SFXWideString text;
    SFBImageSmp image;
    SFCError error;

    if ((error = _calculator->Execute()) == SFERR_NO_ERROR) {
        Apply();
    }
    else {
        switch (error) {
            case SFERR_NO_MEMORY:
                text = "No Memory";
                break;
            case Calculator::ERR_SYNTAX:
                text = "The expression is wrong.";
                break;
            case Calculator::ERR_ZERO_DIVISION:
                text = "0 division. ";
                break;
            case Calculator::ERR_DOMAIN:
                text = "The domain is illegal. ";
                break;
            case Calculator::ERR_PLUS_INF:
                text = "+diverge to infinity";
                break;
            case Calculator::ERR_MINUS_INF:
                text = "-diverge to infinity";
                break;
            case Calculator::ERR_NAN:
                text = "Not an effective value";
                break;
            default:
                text = "The error occurred while operating.";
                break;
        }
        image = SFBShell::GetInstance()->LoadResImage(PSEC_RES_FILE, CANCEL);
        if ((dialog = ::new SFRMessageDialog(psec::CalculateDialogPosition(SFRMessageDialog::GetMargin(), MESSAGEDIALOG_HEIGHT), text, "OK", image)) != null) {
            if (psec::OpenStandardDialog(dialog, HANDLER_FUNCTION(FromSelect)) != SFERR_NO_ERROR) {
                Return();
            }
        }
        else {
            Return();
        }
    }
    return;
}

Void Keyboard::HandlerClear(Void)
{
    _calculator->Edit_clear();
    return;
}

Void Keyboard::HandlerUp(Void)
{
    _calculator->Edit_up();
    return;
}

Void Keyboard::HandlerDown(Void)
{
    _calculator->Edit_down();
    return;
}

Void Keyboard::HandlerLeft(Void)
{
    _calculator->Edit_left();
    return;
}

Void Keyboard::HandlerRight(Void)
{
    _calculator->Edit_right();
    return;
}

Void Keyboard::Return(Void)
{
    KeyboardPtr keyboard;

    if ((keyboard = VNKeyboard::GetInstance()) != null) {
        keyboard->Activate(this);
    }
    return;
}

HANDLER_IMPLEMENT_VOIDRENDER(Keyboard, OnRender, graphics)
{
    static SFXBevelColor::AtomRecConst theme = {
        {0x00, 0x33, 0x33, 0x33},
        {0x00, 0x77, 0x77, 0x77},
        {0x00, 0x99, 0x99, 0x99}
    };
    static SFXRGBColor::AtomRecConst emblem = {
        0x00, 0x55, 0x55, 0x55
    };
    SFRResponderPtr button;
    SFXRectangle rect(GetContentWorld());
    SFXRectangle temp;
    SInt16 i;
    SInt16 j;

    graphics->FillRectangle(rect, theme.base);
    rect.Deflate(0, KEY_MARGIN);
    rect.DivSize(KEYCOUNT_HRIZ, KEYCOUNT_VERT);
    rect.SubY(KEY_MARGIN);
    graphics->FillRectangle(rect, emblem);
    rect.AddY(KEY_MARGIN);
    button = GetFront(TYPE_SFRCONTROL, ATTRIBUTE_WILDCARD, BEHAVIOR_NONE);
    for (j = 0; j < KEYCOUNT_VERT && button != null; ++j) {
        temp = rect;
        temp.Deflate(KEY_MARGIN, 0);
        for (i = 0; i < KEYCOUNT_HRIZ && button != null; ++i) {
            if (button->GetStatusVisible()) {
                graphics->DrawBevelRectangle(temp, theme);
            }
            temp.AddX(rect.GetWidth());
            button = button->GetNext(TYPE_SFRCONTROL, ATTRIBUTE_WILDCARD, BEHAVIOR_NONE, false);
        }
        rect.AddY(rect.GetHeight());
    }
    return;
}

HANDLER_IMPLEMENT_VOIDEVENT(Keyboard, OnSoftkey, event)
{
    switch (event.GetP16()) {
        case Softkey::SOFTKEY_1:
            HandlerSoftkey1();
            break;
        case Softkey::SOFTKEY_2:
            HandlerSoftkey2();
            break;
        case Softkey::SOFTKEY_3:
            HandlerSelect();
            break;
    }
    return;
}

HANDLER_IMPLEMENT_BOOLEVENT(Keyboard, OnKey, event)
{
    SFRResponderPtr button;
    Bool result(false);

    if ((button = GetFront(TYPE_SFRCONTROL, event.GetP16())) != null) {
        if (event.GetType() == SFEVT_KEY_PRESS) {
            button->SetStatusFocus(true);
        }
        result = button->Invoke(SFXEvent(event.GetType(), AVK_SELECT, event.GetP32()));
        if (event.GetType() == SFEVT_KEY_RELEASE) {
            button->SetStatusFocus(false);
        }
    }
    else if (event.GetType() == SFEVT_KEY_PRESS) {
        result = true;
        switch (event.GetP16()) {
            case AVK_SELECT:
                HandlerSelect();
                break;
            case AVK_UP:
                HandlerUp();
                break;
            case AVK_DOWN:
                HandlerDown();
                break;
            case AVK_LEFT:
                HandlerLeft();
                break;
            case AVK_RIGHT:
                HandlerRight();
                break;
            default:
                result = false;
                break;
        }
    }
    return result;
}

HANDLER_IMPLEMENT_VOIDEVENT(Keyboard, OnButton, event)
{
    SFRResponderPtr button;

    if ((button = reinterpret_cast<SFRResponderPtr>(event.GetP32())) != null) {
        switch (button->GetAttribute()) {
            case AVK_CLR:
                HandlerClear();
                break;
            case AVK_1:
                Handler1();
                break;
            case AVK_2:
                Handler2();
                break;
            case AVK_3:
                Handler3();
                break;
            case AVK_4:
                Handler4();
                break;
            case AVK_5:
                Handler5();
                break;
            case AVK_6:
                Handler6();
                break;
            case AVK_7:
                Handler7();
                break;
            case AVK_8:
                Handler8();
                break;
            case AVK_9:
                Handler9();
                break;
            case AVK_STAR:
                HandlerStar();
                break;
            case AVK_0:
                Handler0();
                break;
            case AVK_POUND:
                HandlerPound();
                break;
        }
    }
    return;
}

HANDLER_IMPLEMENT_VOIDEVENT(Keyboard, FromSoftkey1, event)
{
    switch (event.GetP16()) {
        case SRP16_OK:
            psec::CloseStandardDialog(reinterpret_cast<SFRMultiDialogPtr>(event.GetP32()));
            _calculator->Dispose();
            Apply();
            break;
        case SRP16_ESCAPE:
        case SRP16_CANCEL:
            psec::CloseStandardDialog(reinterpret_cast<SFRMultiDialogPtr>(event.GetP32()));
            break;
    }
    return;
}

HANDLER_IMPLEMENT_VOIDEVENT(Keyboard, FromSoftkey2, event)
{
    SFRDialogPtr dialog;

    switch (event.GetP16()) {
        case OPERATION_PREFERENCE:
            psec::CloseStandardMenu(reinterpret_cast<SFRTextMenuPtr>(event.GetP32()));
            if ((dialog = ::new PDialog) != null) {
                psec::OpenCustomDialog(dialog, HANDLER_FUNCTION(FromPreference));
            }
            break;
        case OPERATION_VERSION:
            psec::CloseStandardMenu(reinterpret_cast<SFRTextMenuPtr>(event.GetP32()));
            if ((dialog = ::new VDialog) != null) {
                psec::OpenCustomDialog(dialog, HANDLER_FUNCTION(FromVersion));
            }
            break;
        case SRP16_ESCAPE:
        case SRP16_CANCEL:
            psec::CloseStandardMenu(reinterpret_cast<SFRTextMenuPtr>(event.GetP32()));
            break;
    }
    return;
}

HANDLER_IMPLEMENT_VOIDEVENT(Keyboard, FromSelect, event)
{
    switch (event.GetP16()) {
        case SRP16_OK:
            psec::CloseStandardDialog(reinterpret_cast<SFRMessageDialogPtr>(event.GetP32()));
            Return();
            break;
    }
    return;
}

HANDLER_IMPLEMENT_VOIDEVENT(Keyboard, FromPreference, event)
{
    switch (event.GetP16()) {
        case SRP16_OK:
            psec::CloseCustomDialog(reinterpret_cast<PDialogPtr>(event.GetP32()));
            // TODO : apply config
            break;
        case SRP16_ESCAPE:
        case SRP16_CANCEL:
            psec::CloseCustomDialog(reinterpret_cast<PDialogPtr>(event.GetP32()));
            break;
    }
    return;
}

HANDLER_IMPLEMENT_VOIDEVENT(Keyboard, FromVersion, event)
{
    switch (event.GetP16()) {
        case SRP16_OK:
            psec::CloseCustomDialog(reinterpret_cast<VDialogPtr>(event.GetP32()));
            break;
    }
    return;
}

Void Keyboard::Handler1(Void)
{
    return;
}

Void Keyboard::Handler2(Void)
{
    return;
}

Void Keyboard::Handler3(Void)
{
    return;
}

Void Keyboard::Handler4(Void)
{
    return;
}

Void Keyboard::Handler5(Void)
{
    return;
}

Void Keyboard::Handler6(Void)
{
    return;
}

Void Keyboard::Handler7(Void)
{
    return;
}

Void Keyboard::Handler8(Void)
{
    return;
}

Void Keyboard::Handler9(Void)
{
    return;
}

Void Keyboard::HandlerStar(Void)
{
    return;
}

Void Keyboard::Handler0(Void)
{
    return;
}

Void Keyboard::HandlerPound(Void)
{
    return;
}

Void Keyboard::Apply(Void)
{
    KeyboardPtr keyboard;

    if ((keyboard = MNKeyboard::GetInstance()) != null) {
        keyboard->Refresh();
        keyboard->Activate(this);
    }
    return;
}
