//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      Keyboard.hpp
//

#ifndef __SOPHIACRADLE_KEYBOARD_HPP
#define __SOPHIACRADLE_KEYBOARD_HPP

#include <SophiaFramework.hpp>
#include "Monitor.hpp"
#include "Calculator.hpp"

#define     COLOR_NORMAL        {0x00, 0x00, 0x00, 0x00}
#define     COLOR_DIM           {0x00, 0x33, 0x33, 0x33}
#define     COLOR_POWER         {0x00, 0x00, 0x44, 0x44}
#define     COLOR_MODE          {0x00, 0x44, 0x44, 0x00}
#define     COLOR_SPECIAL       {0x00, 0x44, 0x00, 0x44}
#define     MAP_UNDEFINED       {   "",  COLOR_NORMAL}
#define     MAP_OFF             {"OFF",   COLOR_POWER}
#define     MAP_CNT             {"CNT", COLOR_SPECIAL}
#define     MAP_DEL             {"DEL", COLOR_SPECIAL}
#define     MAP_RTN             {"RTN",    COLOR_MODE}
#define     MAP_VAL             {"VAL",    COLOR_MODE}
#define     MAP_NML             {"NML",    COLOR_MODE}
#define     MAP_SFT             {"SFT",    COLOR_MODE}
#define     MAP_UNT             {"UNT",    COLOR_MODE}
#define     MAP_OPE             {"OPE",    COLOR_MODE}
#define     MAP_FNC             {"FNC",    COLOR_MODE}
#define     MAP_STD             {"STD", COLOR_SPECIAL}
#define     MAP_TRI             {"TRI", COLOR_SPECIAL}
#define     KEY_MARGIN          (2)

SFMTYPEDEFCLASS(Keyboard)

class Keyboard : public SFRPlainWindow {

    SFMSEALCOPY(Keyboard)

    public:

        enum KeyCountEnum {
            KEYCOUNT_HRIZ = 3,
            KEYCOUNT_VERT = 5
        };

    public:

        SFMTYPEDEFSTRUCT(StateRec)
        struct StateRec {
            AChar title[6];
            SFXRGBColor::AtomRec color;
        };
        SFMTYPEDEFSTRUCT(MapRec)
        struct MapRec {
            StateRec state[KEYCOUNT_VERT][KEYCOUNT_HRIZ];
        };

    private:

        SFMTYPEDEFSTRUCT(KeyRec)
        struct KeyRec {
            AVKType event[KEYCOUNT_VERT][KEYCOUNT_HRIZ];
        };

    private:

        MonitorPtr _monitor;
        CalculatorPtr _calculator;

    public:

        virtual ~Keyboard(Void);
        virtual Void Refresh(Void) = null;
        virtual Void Activate(KeyboardPtr inactivate) = null;

    protected:

        explicit Keyboard(SFRApplicationPtr director, SFXRectangleConstRef rect, SFCType attribute, MonitorPtr monitor, CalculatorPtr calculator) static_throws;
        static KeyboardPtr GetInstance(SFCType attribute);
        CalculatorPtr GetCalculator(Void) const;
        Void Refresh(MapRecConstRef map);
        Void Activate(KeyboardPtr inactivate, UInt32 mode);
        virtual Void HandlerSoftkey1(Void);
        virtual Void HandlerSoftkey2(Void);
        virtual Void HandlerSelect(Void);
        virtual Void HandlerClear(Void);
        virtual Void HandlerUp(Void);
        virtual Void HandlerDown(Void);
        virtual Void HandlerLeft(Void);
        virtual Void HandlerRight(Void);
        Void Return(Void);

    private:

        HANDLER_DECLARE_VOIDRENDER(OnRender)
        HANDLER_DECLARE_VOIDEVENT(OnSoftkey)
        HANDLER_DECLARE_BOOLEVENT(OnKey)
        HANDLER_DECLARE_VOIDEVENT(OnButton)
        HANDLER_DECLARE_VOIDEVENT(FromSoftkey1)
        HANDLER_DECLARE_VOIDEVENT(FromSoftkey2)
        HANDLER_DECLARE_VOIDEVENT(FromSelect)
        HANDLER_DECLARE_VOIDEVENT(FromPreference)
        HANDLER_DECLARE_VOIDEVENT(FromVersion)
        virtual Void Handler1(Void);
        virtual Void Handler2(Void);
        virtual Void Handler3(Void);
        virtual Void Handler4(Void);
        virtual Void Handler5(Void);
        virtual Void Handler6(Void);
        virtual Void Handler7(Void);
        virtual Void Handler8(Void);
        virtual Void Handler9(Void);
        virtual Void HandlerStar(Void);
        virtual Void Handler0(Void);
        virtual Void HandlerPound(Void);
        Void Apply(Void);
};

inline CalculatorPtr Keyboard::GetCalculator(Void) const
{
    return _calculator;
}

#endif // __SOPHIACRADLE_KEYBOARD_HPP //
