//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      Monitor.cpp
//

#include "Monitor.hpp"
#include "psec_res.h"

#define     FLUSH_INTERVAL      (500)
#define     ANSWER_MARGIN       (8)
#define     BLOCK_SIZE          (8)
#define     BLOCK_LENGTH        (8)

Monitor::Monitor(SFRApplicationPtr director, SFXRectangleConstRef rect) static_throws : SFRPlainWindow(director, rect, BEHAVIOR_SFRPLAINWINDOW, ATTRIBUTE_MONITOR)
{
    SInt16 temp;

    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnRender)));
        if (static_try()) {
            static_throw(RegisterHandler(SFEVT_APP_RESUME, HANDLER_BEFORE, HANDLER_FUNCTION(OnResume)));
            if (static_try()) {
                static_throw(RegisterHandler(SFEVT_APP_SUSPEND, HANDLER_AFTER, HANDLER_FUNCTION(OnSuspend)));
                if (static_try()) {
                    if ((_bitmap = SFBShell::GetInstance()->LoadResBitmap(PSEC_RES_FILE, MODE)) != null) {
                        _device = GetContentWorld();
                        _device.Deflate(2, 2);
                        _matrix.Set(SFXGraphics::MeasureText(AEE_FONT_NORMAL, "W") + 4, SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) + 4);
                        _size.SetWidth(_device.GetWidth() / _matrix.GetWidth());
                        temp = (_device.GetWidth() - _matrix.GetWidth() * _size.GetWidth()) / 2;
                        _device.Deflate(temp, temp);
                        _size.SetHeight((_device.GetHeight() - _matrix.GetHeight() - ANSWER_MARGIN * 2) / _matrix.GetHeight());
                        _formula = "";
                        _answer = "";
                        _number = Calculator::NUMBERTYPE_LIMIT;
                        _sign = Calculator::SIGNTYPE_LIMIT;
                        _bit = Calculator::BITTYPE_LIMIT;
                        _angle = Calculator::ANGLETYPE_LIMIT;
                        _mode = MODE_NONE;
                        _offset = 0;
                        _cursor = 0;
                        _length = 1;
                        _maximum = 0;
                        _block = 0;
                        StartTimer();
                    }
                    else {
                        static_throw(SFERR_FAILED);
                    }
                }
            }
        }
    }
    return;
}

Monitor::~Monitor(Void)
{
    StopTimer();
    return;
}

MonitorPtr Monitor::GetInstance(Void)
{
    return static_cast<MonitorPtr>(SFRApplication::GetInstance()->GetFront(TYPE_SFRWINDOW, ATTRIBUTE_MONITOR, BEHAVIOR_NONE));
}

Void Monitor::SetFormula(SFXAnsiStringConstRef param)
{
    if (param != _formula) {
        _formula = param;
        AdjustCursor();
        Invalidate();
    }
    return;
}

Void Monitor::SetAnswer(SFXAnsiStringConstRef param)
{
    if (param != _answer) {
        _answer = param;
        AdjustBlock();
        Invalidate();
    }
    return;
}

Void Monitor::SetNumberType(Calculator::NumberTypeEnum param)
{
    if (param != _number) {
        _number = param;
        Invalidate();
    }
    return;
}

Void Monitor::SetSignType(Calculator::SignTypeEnum param)
{
    if (param != _sign) {
        _sign = param;
        Invalidate();
    }
    return;
}

Void Monitor::SetBitType(Calculator::BitTypeEnum param)
{
    if (param != _bit) {
        _bit = param;
        Invalidate();
    }
    return;
}

Void Monitor::SetAngleType(Calculator::AngleTypeEnum param)
{
    if (param != _angle) {
        _angle = param;
        Invalidate();
    }
    return;
}

Void Monitor::SetMode(UInt32 param)
{
    if (param != _mode) {
        _mode = param;
        Invalidate();
    }
    return;
}

Void Monitor::SetCursor(SInt16 cursor, SInt16 length)
{
    SInt16 scursor;
    SInt16 slength;

    if (cursor != _cursor || length != _length) {
        scursor = _cursor;
        slength = _length;
        _cursor = cursor;
        _length = length;
        AdjustCursor();
        if (_cursor != scursor || _length != slength) {
            Invalidate();
        }
    }
    return;
}

Void Monitor::SetBlock(SInt16 block)
{
    SInt16 sblock;

    if (block != _block) {
        sblock = _block;
        _block = block;
        AdjustBlock();
        if (_block != sblock) {
            Invalidate();
        }
    }
    return;
}

HANDLER_IMPLEMENT_VOIDRENDER(Monitor, OnRender, graphics)
{
    static SFXBevelColor::AtomRecConst theme = {
        {0x00, 0xFF, 0xFF, 0xFF},
        {0x00, 0xE7, 0xEB, 0xC9},
        {0x00, 0xC0, 0xC2, 0x9F}
    };
    static SFXRGBColor::AtomRecConst digit[] = {
        {0x00, 0x00, 0x00, 0x00},
        {0x00, 0xD6, 0xDA, 0xB8},
        {0x00, 0xD6, 0xDA, 0xB8},
        {0x00, 0x4E, 0x52, 0x30}
    };
    static SFXRectangle::AtomRecConst number[] = {
        { 0,  0, 20,  9}, {20,  0, 20,  9}, {40,  0, 20,  9}, {60,  0, 20,  9}, {80,  0, 20,  9}
    };
    static SFXRectangle::AtomRecConst angle[] = {
        { 0,  9, 21,  9}, {21,  9, 21,  9}, {42,  9, 21,  9}, {63,  9, 21,  9}
    };
    static SFXRectangle::AtomRecConst sign[] = {
        { 0, 18,  9,  9}, { 9, 18,  9,  9}, {18, 18,  9,  9}
    };
    static SFXRectangle::AtomRecConst bit[] = {
        { 0, 27, 15,  9}, {15, 27, 15,  9}, {30, 27, 15,  9}, {45, 27, 15,  9}, {60, 27, 15,  9}
    };
    static SFXRectangle::AtomRecConst block[] = {
        {27, 18, 30,  9}, {57, 18, 30,  9}
    };
    static SFXRectangle::AtomRecConst formula[] = {
        { 0, 36, 43,  9}, {43, 36, 43,  9}
    };
    static SFXRectangle::AtomRecConst answer[] = {
        { 0, 45, 38,  9}, {38, 45, 38,  9}
    };
    static SFXRectangle::AtomRecConst mode[][2] = {
        {{ 0, 54,  9,  9}, { 9, 54,  9,  9}},
        {{18, 54,  9,  9}, {27, 54,  9,  9}},
        {{36, 54,  9,  9}, {45, 54,  9,  9}},
        {{54, 54,  9,  9}, {63, 54,  9,  9}},
        {{72, 54,  9,  9}, {81, 54,  9,  9}}
    };
    SFXRectangle horizontal;
    SFXRectangle vertical;
    SFXTriangle indicator;
    SFXWideString string;
    SFXWideString text;
    SInt16 index;
    SInt16 i;
    SInt16 j;

    graphics->FillBevelRectangle(GetContentWorld(), theme);
    vertical.Set(_device.GetOrigin(), _matrix);
    index = _offset * _size.GetWidth();
    for (j = 0; j < _size.GetHeight(); ++j) {
        horizontal = vertical;
        horizontal.Deflate(1, 1);
        for (i = 0; i < _size.GetWidth(); ++i, ++index) {
            text.Set((0 <= index && index < _formula.GetLength()) ? (_formula[index]) : (' '));
            if (_maximum < 1 && _flush && _cursor <= index && index <= _cursor + _length - 1) {
                graphics->SetForeColor(digit[2]);
                graphics->SetBackColor(digit[3]);
            }
            else {
                graphics->SetForeColor(digit[0]);
                graphics->SetBackColor(digit[1]);
            }
            graphics->DrawText(text, horizontal, IDF_RECT_FILL);
            horizontal.AddX(_matrix.GetWidth());
        }
        vertical.AddY(_matrix.GetHeight());
    }
    horizontal.Set(0, 0, 11, 7);
    horizontal.SnapRightTop(_device.GetRight() - 2, vertical.GetTop());
    indicator.Set(horizontal.GetVertexLeftTop().Offset(1, 1), horizontal.GetVertexLeftBottom().Offset(horizontal.GetWidth() / 2, -1), horizontal.GetVertexRightTop().Offset(-1, 1));
    graphics->FillTriangle(indicator, digit[(_offset <= _formula.GetLength() / _size.GetWidth() - _size.GetHeight()) ? (3) : (1)]);
    horizontal.SubX(horizontal.GetWidth());
    indicator.Set(horizontal.GetVertexLeftBottom().Offset(1, -1), horizontal.GetVertexLeftTop().Offset(horizontal.GetWidth() / 2, 1), horizontal.GetVertexRightBottom().Offset(-1, -1));
    graphics->FillTriangle(indicator, digit[(_offset > 0) ? (3) : (1)]);
    horizontal.SubOrigin(mode[0][0].size.width, 1);
    for (i = MODE_LIMIT - 1; i >= 0; --i) {
        horizontal.SubX(horizontal.GetWidth());
        horizontal.SetSize(mode[i][0].size);
        graphics->BitBlt(horizontal, _bitmap, mode[i][(_mode & (0x01 << i)) ? (1) : (0)].origin);
    }
    horizontal = formula[0];
    horizontal.SnapLeftTop(_device.GetLeft() - 1, vertical.GetTop() - 1);
    graphics->BitBlt(horizontal, _bitmap, formula[(_maximum < 1) ? (1) : (0)].origin);
    horizontal.AddX(horizontal.GetWidth());
    horizontal.SetSize(answer[0].size);
    graphics->BitBlt(horizontal, _bitmap, answer[(_maximum > 0) ? (1) : (0)].origin);
    string = _answer;
    if (_maximum > 1) {
        index = static_cast<SInt16>(string.GetLength()) - (_block + 1) * BLOCK_LENGTH;
        string = string.Substring(index, index + BLOCK_LENGTH);
    }
    graphics->SetForeColor(digit[0]);
    graphics->SetBackColor(digit[1]);
    horizontal.Set(0, 0, _matrix);
    horizontal.SnapLeftBottom(_device.GetLeft(), _device.GetBottom() - ANSWER_MARGIN);
    horizontal.Deflate(1, 1);
    index = static_cast<SInt16>(string.GetLength()) - _size.GetWidth();
    for (i = 0; i < _size.GetWidth(); ++i, ++index) {
        text.Set((0 <= index && index < string.GetLength()) ? (string[index]) : (' '));
        graphics->DrawText(text, horizontal, IDF_RECT_FILL);
        horizontal.AddX(_matrix.GetWidth());
    }
    vertical.Set(0, 0, 10, 7);
    vertical.SnapRightBottom(_device.GetRightBottom());
    horizontal = vertical;
    horizontal.Deflate(1, 1);
    for (i = 0; i < BLOCK_SIZE; ++i) {
        graphics->FillRectangle(horizontal, digit[1]);
        if (i < _maximum) {
            if (_flush && i == _block) {
                graphics->FillRectangle(horizontal, digit[3]);
            }
            else if (_maximum > 1) {
                graphics->DrawRectangle(horizontal, digit[3]);
            }
        }
        horizontal.SubX(vertical.GetWidth());
    }
    horizontal.Offset(0, 2);
    vertical = block[0];
    vertical.SnapRightBottom(horizontal.GetRightBottom());
    graphics->BitBlt(vertical, _bitmap, block[(_maximum > 1) ? (1) : (0)].origin);
    horizontal = number[0];
    horizontal.SnapLeftBottom(_device.GetLeft() - 1, vertical.GetBottom());
    graphics->BitBlt(horizontal, _bitmap, number[(_number < Calculator::NUMBERTYPE_LIMIT) ? (_number + 1) : (0)].origin);
    horizontal.AddX(horizontal.GetWidth());
    horizontal.SetSize(angle[0].size);
    graphics->BitBlt(horizontal, _bitmap, angle[(_number == Calculator::NUMBERTYPE_DEC && _angle < Calculator::ANGLETYPE_LIMIT) ? (_angle + 1) : (0)].origin);
    horizontal.AddX(horizontal.GetWidth());
    horizontal.SetSize(sign[0].size);
    graphics->BitBlt(horizontal, _bitmap, sign[(_number != Calculator::NUMBERTYPE_DEC && _sign < Calculator::SIGNTYPE_LIMIT) ? (_sign + 1) : (0)].origin);
    horizontal.AddX(horizontal.GetWidth());
    horizontal.SetSize(bit[0].size);
    graphics->BitBlt(horizontal, _bitmap, bit[(_number != Calculator::NUMBERTYPE_DEC && _bit < Calculator::BITTYPE_LIMIT) ? (_bit + 1) : (0)].origin);
    return;
}

HANDLER_IMPLEMENT_VOIDRESUME(Monitor, OnResume, environment)
{
    unused(environment);

    StartTimer();
    return;
}

HANDLER_IMPLEMENT_VOIDSUSPEND(Monitor, OnSuspend, reason, info)
{
    unused(reason);
    unused(info);

    StopTimer();
    return;
}

Void Monitor::Invalidate(Void)
{
    _timer.Cancel();
    _flush = true;
    InvalidateContent(_device);
    _timer.Schedule(FLUSH_INTERVAL);
    return;
}

Void Monitor::StartTimer(Void)
{
    _flush = true;
    _timer.Set(CALLBACK_FUNCTION(OnTimer));
    _timer.Schedule(FLUSH_INTERVAL);
    return;
}

Void Monitor::StopTimer(Void)
{
    _timer.Cancel();
    return;
}

CALLBACK_IMPLEMENT_SFXTIMER(Monitor, OnTimer)
{
    static SFXEvent::AtomRecConst event = {
        SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, false
    };

    _flush = !_flush;
    InvalidateContent(_device);
    SFRApplication::GetInstance()->Invoke(event);
    _timer.Schedule(FLUSH_INTERVAL);
    return;
}

Void Monitor::AdjustCursor(Void)
{
    if (_length < 1) {
        _length = 1;
    }
    if (_cursor < 0) {
        _cursor = 0;
    }
    else if (_cursor + _length - 1 > _formula.GetLength()) {
        _cursor = static_cast<SInt16>(_formula.GetLength()) - _length + 1;
    }
    if (_cursor < _offset * _size.GetWidth()) {
        _offset = _cursor / _size.GetWidth();
    }
    else if (_cursor + _length > (_offset + _size.GetHeight()) * _size.GetWidth()) {
        _offset = (_cursor + _length - 1) / _size.GetWidth() - _size.GetHeight() + 1;
    }
    return;
}

Void Monitor::AdjustBlock(Void)
{
    if (_answer.IsEmpty()) {
        _maximum = 0;
        _block = 0;
    }
    else if (_answer.GetLength() <= _size.GetWidth()) {
        _maximum = 1;
        _block = 0;
    }
    else {
        _maximum = static_cast<SInt16>(_answer.GetLength()) / BLOCK_LENGTH;
        if (_answer.GetLength() % BLOCK_LENGTH > 0) {
            ++_maximum;
        }
        if (_block < 0) {
            _block = 0;
        }
        else if (_block >= _maximum) {
            _block = _maximum - 1;
        }
    }
    return;
}
