//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      Monitor.hpp
//

#ifndef __SOPHIACRADLE_MONITOR_HPP
#define __SOPHIACRADLE_MONITOR_HPP

#include <SophiaFramework.hpp>
#include "Calculator.hpp"

SFMRESPONDERATTRIBUTE(MONITOR, four_char_code('M', 'n', 't', 'r'))

SFMTYPEDEFCLASS(Monitor)

class Monitor : public SFRPlainWindow {

    SFMSEALCOPY(Monitor)

    public:
        enum ModeEnum {
            MODE_NONE = 0x00 << 0,
            MODE_SHIFT = 0x01 << 0,
            MODE_UNIT = 0x01 << 1,
            MODE_OPERATOR = 0x01 << 2,
            MODE_FUNCTION = 0x01 << 3,
            MODE_TRIGONOMETRIC = 0x01 << 4,
            MODE_LIMIT = 5
        };

    private:

        SFBBitmapSmp _bitmap;
        SFXRectangle _device;
        SFXSize _matrix;
        SFXSize _size;
        SFXWideString _formula;
        SFXWideString _answer;
        Calculator::NumberTypeEnum _number;
        Calculator::SignTypeEnum _sign;
        Calculator::BitTypeEnum _bit;
        Calculator::AngleTypeEnum _angle;
        UInt32 _mode;
        SInt16 _offset;
        SInt16 _cursor;
        SInt16 _length;
        SInt16 _maximum;
        SInt16 _block;
        SFXTimer _timer;
        Bool _flush;

    public:

        explicit Monitor(SFRApplicationPtr director, SFXRectangleConstRef rect) static_throws;
        virtual ~Monitor(Void);

        static MonitorPtr GetInstance(Void);
        SFXSizeConstRef GetSize(Void) const;
        Void SetFormula(SFXAnsiStringConstRef param);
        SFXAnsiString GetFormula(Void) const;
        Void SetAnswer(SFXAnsiStringConstRef param);
        SFXAnsiString GetAnswer(Void) const;
        Void SetNumberType(Calculator::NumberTypeEnum param);
        Void SetSignType(Calculator::SignTypeEnum param);
        Void SetBitType(Calculator::BitTypeEnum param);
        Void SetAngleType(Calculator::AngleTypeEnum param);
        Void SetMode(UInt32 param);
        Void SetCursor(SInt16 cursor, SInt16 length = 1);
        SInt16 GetCursor(Void) const;
        Void SetBlock(SInt16 block);
        SInt16 GetBlock(Void) const;

    private:

        HANDLER_DECLARE_VOIDRENDER(OnRender)
        HANDLER_DECLARE_VOIDRESUME(OnResume)
        HANDLER_DECLARE_VOIDSUSPEND(OnSuspend)
        Void Invalidate(Void);
        Void StartTimer(Void);
        Void StopTimer(Void);
        CALLBACK_DECLARE_SFXTIMER(OnTimer)
        Void AdjustCursor(Void);
        Void AdjustBlock(Void);
};

inline SFXSizeConstRef Monitor::GetSize(Void) const
{
    return _size;
}

inline SFXAnsiString Monitor::GetFormula(Void) const
{
    return _formula;
}

inline SFXAnsiString Monitor::GetAnswer(Void) const
{
    return _answer;
}

inline SInt16 Monitor::GetCursor(Void) const
{
    return _cursor;
}

inline SInt16 Monitor::GetBlock(Void) const
{
    return _block;
}

#endif // __SOPHIACRADLE_MONITOR_HPP //
