//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      VSKeyboard.cpp
//

#include "VSKeyboard.hpp"
#include "VUKeyboard.hpp"
#include "ONKeyboard.hpp"

VSKeyboard::VSKeyboard(SFRApplicationPtr director, SFXRectangleConstRef rect, MonitorPtr monitor, CalculatorPtr calculator) static_throws : Keyboard(director, rect, ATTRIBUTE_VSKEYBOARD, monitor, calculator)
{
    if (static_try()) {
        Refresh();
    }
    return;
}

VSKeyboardPtr VSKeyboard::GetInstance(Void)
{
    return static_cast<VSKeyboardPtr>(Keyboard::GetInstance(ATTRIBUTE_VSKEYBOARD));
}

Void VSKeyboard::HandlerClear(Void)
{
    Return();
    return;
}

Void VSKeyboard::Handler1(Void)
{
    switch (GetCalculator()->GetNumberType()) {
        case Calculator::NUMBERTYPE_DEC:
            GetCalculator()->Number_pi();
            Return();
            break;
        case Calculator::NUMBERTYPE_HEX:
            GetCalculator()->Number_A();
            break;
    }
    return;
}

Void VSKeyboard::Handler2(Void)
{
    switch (GetCalculator()->GetNumberType()) {
        case Calculator::NUMBERTYPE_DEC:
            GetCalculator()->Number_e();
            Return();
            break;
        case Calculator::NUMBERTYPE_HEX:
            GetCalculator()->Number_B();
            break;
    }
    return;
}

Void VSKeyboard::Handler3(Void)
{
    switch (GetCalculator()->GetNumberType()) {
        case Calculator::NUMBERTYPE_HEX:
            GetCalculator()->Number_C();
            break;
    }
    return;
}

Void VSKeyboard::Handler4(Void)
{
    switch (GetCalculator()->GetNumberType()) {
        case Calculator::NUMBERTYPE_HEX:
            GetCalculator()->Number_D();
            break;
    }
    return;
}

Void VSKeyboard::Handler5(Void)
{
    switch (GetCalculator()->GetNumberType()) {
        case Calculator::NUMBERTYPE_HEX:
            GetCalculator()->Number_E();
            break;
    }
    return;
}

Void VSKeyboard::Handler6(Void)
{
    switch (GetCalculator()->GetNumberType()) {
        case Calculator::NUMBERTYPE_DEC:
            GetCalculator()->Number_Ran();
            Return();
            break;
        case Calculator::NUMBERTYPE_HEX:
            GetCalculator()->Number_F();
            break;
    }
    return;
}

Void VSKeyboard::Handler7(Void)
{
    GetCalculator()->Parenthesis_left();
    Return();
    return;
}

Void VSKeyboard::Handler8(Void)
{
    GetCalculator()->Parenthesis_right();
    Return();
    return;
}

Void VSKeyboard::Handler9(Void)
{
    GetCalculator()->Number_Ans();
    Return();
    return;
}

Void VSKeyboard::HandlerStar(Void)
{
    KeyboardPtr keyboard;

    if ((keyboard = VUKeyboard::GetInstance()) != null) {
        keyboard->Activate(this);
    }
    return;
}

Void VSKeyboard::Handler0(Void)
{
    switch (GetCalculator()->GetNumberType()) {
        case Calculator::NUMBERTYPE_DEC:
            GetCalculator()->Number_period();
            Return();
            break;
    }
    return;
}

Void VSKeyboard::HandlerPound(Void)
{
    KeyboardPtr keyboard;

    if ((keyboard = ONKeyboard::GetInstance()) != null) {
        keyboard->Activate(this);
    }
    return;
}

Void VSKeyboard::Refresh(Void)
{
    static MapRecConst map[] = {
        {{// 02
            {       MAP_UNDEFINED,             MAP_RTN,                 MAP_OFF},
            {       MAP_UNDEFINED,       MAP_UNDEFINED,           MAP_UNDEFINED},
            {       MAP_UNDEFINED,       MAP_UNDEFINED,           MAP_UNDEFINED},
            {{ "(", COLOR_NORMAL}, {")", COLOR_NORMAL}, { "Ans", COLOR_SPECIAL}},
            {       MAP_UNDEFINED,       MAP_UNDEFINED,                 MAP_OPE}
        }},
        {{// 08
            {       MAP_UNDEFINED,             MAP_RTN,                 MAP_OFF},
            {       MAP_UNDEFINED,       MAP_UNDEFINED,           MAP_UNDEFINED},
            {       MAP_UNDEFINED,       MAP_UNDEFINED,           MAP_UNDEFINED},
            {{ "(", COLOR_NORMAL}, {")", COLOR_NORMAL}, { "Ans", COLOR_SPECIAL}},
            {       MAP_UNDEFINED,       MAP_UNDEFINED,                 MAP_OPE}
        }},
        {{// 10
            {       MAP_UNDEFINED,             MAP_RTN,                 MAP_OFF},
            {{"pi", COLOR_NORMAL}, {"e", COLOR_NORMAL},           MAP_UNDEFINED},
            {       MAP_UNDEFINED,       MAP_UNDEFINED, {"Ran#",  COLOR_NORMAL}},
            {{ "(", COLOR_NORMAL}, {")", COLOR_NORMAL}, { "Ans", COLOR_SPECIAL}},
            {             MAP_UNT, {".", COLOR_NORMAL},                 MAP_OPE}
        }},
        {{// 16
            {       MAP_UNDEFINED,             MAP_RTN,                 MAP_OFF},
            {{ "A", COLOR_NORMAL}, {"B", COLOR_NORMAL}, {   "C",  COLOR_NORMAL}},
            {{ "D", COLOR_NORMAL}, {"E", COLOR_NORMAL}, {   "F",  COLOR_NORMAL}},
            {{ "(", COLOR_NORMAL}, {")", COLOR_NORMAL}, { "Ans", COLOR_SPECIAL}},
            {       MAP_UNDEFINED,       MAP_UNDEFINED,                 MAP_OPE}
        }}
    };

    Keyboard::Refresh(map[GetCalculator()->GetNumberType()]);
    return;
}

Void VSKeyboard::Activate(KeyboardPtr inactivate)
{
    Keyboard::Activate(inactivate, Monitor::MODE_SHIFT);
    return;
}
