//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      psec.cpp
//

#include "psec.hpp"
#include "psec_res.h"
#include "Softkey.hpp"
#include "Monitor.hpp"
#include "MNKeyboard.hpp"
#include "VNKeyboard.hpp"
#include "VSKeyboard.hpp"
#include "VUKeyboard.hpp"
#include "ONKeyboard.hpp"
#include "FNKeyboard.hpp"
#include "FSKeyboard.hpp"
#include "TNKeyboard.hpp"
#include "TSKeyboard.hpp"

#define     INVALID_MEMORY_SIZE     (320000)
#define     HORIZONTAL_MARGIN       (9)
#define     VERTICAL_MARGIN         (6)
#define     FONT_MARGIN             (5)
#define     MONITOR_MARGIN          (2)
#define     DIALOG_MARGIN           (12)
#define     MENU_MARGIN             (2)
#define     LABEL_SOFTKEY2          ("Close")
#define     LABEL_SOFTKEY3          ("Select")
#define     MESSAGEDIALOG_HEIGHT    (104)

SFCApplet::FactorySPP SFCApplet::Boot(AEECLSID id, SFXAnsiStringPtr license)
{
    *license = "heap://"
        "R9YQSEBSLXM3Y9T7UJRGXHWAXEXBTCXMDV0SEU8ZCV8U8VOU29W2YIR0TNV0YK9YIR2U5X6Z5UPTDT4W"
        "HYPXOR0W9V4QU9SKV1SQSIQVEZ6SKXBT3YFWMZ3U8SQY4TLSOTDSLR3DUGO9W9U2VBZBYPWBWHT1UMSP"
        "YCZ95UCLTQT2V4TMYB4RJ2U9YPRG34X3VCVCVFSJSKV7Z0VKSOTCYLXISDVCRNYA86IT0YMUIVAYMTB1"
        "T3Z7U2R7VLT4SQR0W4WBZ8V1UPEV5TQR10V5Z6UFWMYDX8XMW4T51W2UOSLXPTOVDXGUMVIRCY2U0Y4Y"
        "5XOWLYPTISEWJZ31Z0RPXDNWIWMT5RBV3RJTDTF"
    "/";

    return (id == AEECLSID_PSEC) ? (&psec::Factory) : (null);
}

SFCInvokerPtr psec::Factory(Void)
{
    SFCInvokerPtr result(null);

    #if TARGET_VERSION_GE(4, 0, 0)
    result = ::new psec;
    #else
    SFBHeapSmp heap;
    if ((heap = SFBHeap::NewInstance()) != null) {
        if (heap->CheckAvail(INVALID_MEMORY_SIZE)) {
            result = ::new psec;
        }
    }
    #endif
    return result;

}

psec::psec(Void) static_throws : _calculator(null)
{
    SoftkeyPtr softkey;
    MonitorPtr monitor;
    KeyboardPtr keyboard;
    SFXRectangle rect;
    SFXRectangle temp;

    if (static_try()) {
#if defined TARGET_EXTENSION_KDDI
#if defined TARGET_ENVIRONMENT_PHYSICAL
#if TARGET_VERSION_LT(3, 1, 0)
        _native = null;
        if ((_font = SFBFont::NewInstance(AEECLSID_KDDI_SMALL_FONT)) != null) {
            _native = SFXGraphics::GetInstance()->GetDisplay()->SetFont(AEE_FONT_NORMAL, _font.Get());
        }
        else {
            static_throw(SFERR_NO_MEMORY);
        }
#endif
#endif
#endif
        if (static_try()) {
            static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnRender)));
            if (static_try()) {
                static_throw(RegisterHandler(SFEVT_APP_START, HANDLER_BEFORE, HANDLER_FUNCTION(OnStart)));
                if (static_try()) {
                    static_throw(RegisterHandler(SFEVT_APP_STOP, HANDLER_AFTER, HANDLER_FUNCTION(OnStop)));
                    if (static_try()) {
                        if ((softkey = ::new Softkey(this)) != null) {
                            static_throw(*softkey);
                            if (static_try()) {
                                rect = GetContentWorld();
                                rect.SetBottom(softkey->GetBaseBound().GetTop());
                                rect.Deflate(HORIZONTAL_MARGIN, VERTICAL_MARGIN);
                                temp = rect;
                                temp.SetTop(temp.GetBottom() - (SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) + FONT_MARGIN) * Keyboard::KEYCOUNT_VERT);
                                rect.SetBottom(temp.GetTop() - VERTICAL_MARGIN);
                                rect.Deflate(MONITOR_MARGIN, MONITOR_MARGIN);
                                temp.Inflate(0, KEY_MARGIN);
                                if ((monitor = ::new Monitor(this, rect)) != null) {
                                    static_throw(*monitor);
                                    if (static_try()) {
                                        if ((_calculator = ::new Calculator(monitor)) != null) {
                                            if ((keyboard = ::new TSKeyboard(this, temp, monitor, _calculator)) != null) {
                                                static_throw(*keyboard);
                                                if (static_try()) {
                                                    if ((keyboard = ::new TNKeyboard(this, temp, monitor, _calculator)) != null) {
                                                        static_throw(*keyboard);
                                                        if (static_try()) {
                                                            if ((keyboard = ::new FSKeyboard(this, temp, monitor, _calculator)) != null) {
                                                                static_throw(*keyboard);
                                                                if (static_try()) {
                                                                    if ((keyboard = ::new FNKeyboard(this, temp, monitor, _calculator)) != null) {
                                                                        static_throw(*keyboard);
                                                                        if (static_try()) {
                                                                            if ((keyboard = ::new ONKeyboard(this, temp, monitor, _calculator)) != null) {
                                                                                static_throw(*keyboard);
                                                                                if (static_try()) {
                                                                                    if ((keyboard = ::new VUKeyboard(this, temp, monitor, _calculator)) != null) {
                                                                                        static_throw(*keyboard);
                                                                                        if (static_try()) {
                                                                                            if ((keyboard = ::new VSKeyboard(this, temp, monitor, _calculator)) != null) {
                                                                                                static_throw(*keyboard);
                                                                                                if (static_try()) {
                                                                                                    if ((keyboard = ::new VNKeyboard(this, temp, monitor, _calculator)) != null) {
                                                                                                        static_throw(*keyboard);
                                                                                                        if (static_try()) {
                                                                                                            if ((keyboard = ::new MNKeyboard(this, temp, monitor, _calculator)) != null) {
                                                                                                                static_throw(*keyboard);
                                                                                                                if (!static_try()) {
                                                                                                                    ::delete keyboard;
                                                                                                                }
                                                                                                            }
                                                                                                            else {
                                                                                                                static_throw(SFERR_NO_MEMORY);
                                                                                                            }
                                                                                                        }
                                                                                                        else {
                                                                                                            ::delete keyboard;
                                                                                                        }
                                                                                                    }
                                                                                                    else {
                                                                                                        static_throw(SFERR_NO_MEMORY);
                                                                                                    }
                                                                                                }
                                                                                                else {
                                                                                                    ::delete keyboard;
                                                                                                }
                                                                                            }
                                                                                            else {
                                                                                                static_throw(SFERR_NO_MEMORY);
                                                                                            }
                                                                                        }
                                                                                        else {
                                                                                            ::delete keyboard;
                                                                                        }
                                                                                    }
                                                                                    else {
                                                                                        static_throw(SFERR_NO_MEMORY);
                                                                                    }
                                                                                }
                                                                                else {
                                                                                    ::delete keyboard;
                                                                                }
                                                                            }
                                                                            else {
                                                                                static_throw(SFERR_NO_MEMORY);
                                                                            }
                                                                        }
                                                                        else {
                                                                            ::delete keyboard;
                                                                        }
                                                                    }
                                                                    else {
                                                                        static_throw(SFERR_NO_MEMORY);
                                                                    }
                                                                }
                                                                else {
                                                                    ::delete keyboard;
                                                                }
                                                            }
                                                            else {
                                                                static_throw(SFERR_NO_MEMORY);
                                                            }
                                                        }
                                                        else {
                                                            ::delete keyboard;
                                                        }
                                                    }
                                                    else {
                                                        static_throw(SFERR_NO_MEMORY);
                                                    }
                                                }
                                                else {
                                                    ::delete keyboard;
                                                }
                                            }
                                            else {
                                                static_throw(SFERR_NO_MEMORY);
                                            }
                                        }
                                        else {
                                            static_throw(SFERR_NO_MEMORY);
                                        }
                                    }
                                    else {
                                        ::delete monitor;
                                    }
                                }
                                else {
                                    static_throw(SFERR_NO_MEMORY);
                                }
                            }
                            else {
                                ::delete softkey;
                            }
                        }
                        else {
                            static_throw(SFERR_NO_MEMORY);
                        }
                    }
                }
            }
        }
    }
    return;
}

psec::~psec(Void)
{
    if (_calculator != null) {
        ::delete _calculator;
    }
#if defined TARGET_ENVIRONMENT_PHYSICAL
#if TARGET_VERSION_LT(3, 1, 0)
    if (_native != null) {
        SFXGraphics::GetInstance()->GetDisplay()->SetFont(AEE_FONT_NORMAL, _native);
    }
#endif
#endif
    return;
}

SFXRectangle psec::CalculateWindowPosition(Void)
{
    SoftkeyPtr softkey;
    SFXRectangle result(GetInstance()->GetContentWorld());

    if ((softkey = Softkey::GetInstance()) != null) {
        result.SubBottom(softkey->GetBaseBound().GetHeight());
    }
    return result;
}

SFXRectangle psec::CalculateDialogPosition(SFXMarginConstRef margin, SInt16 height)
{
    SoftkeyPtr softkey;
    SFXRectangle rect(GetInstance()->GetContentWorld());
    SFXRectangle result(0, 0, 0, 0);

    result.SetSize(rect.GetWidth() - DIALOG_MARGIN * 2 - margin.GetLeft() - margin.GetRight(), height);
    result.Offset(SFXGrid(rect.GetSize() - result.GetSize()) / 2);
    result.Inflate(margin);
    result.AddY(margin.GetTop() - margin.GetBottom());
    if ((softkey = Softkey::GetInstance()) != null) {
        result.SubY(softkey->GetBaseBound().GetHeight());
    }
    return result;
}

SFXRectangle psec::CalculateMenuPosition(SInt16 limit)
{
    SoftkeyPtr softkey;
    SFXRectangle rect(GetInstance()->GetContentWorld());
    SInt16 height;
    SFXRectangle result(0, 0, 0, 0);

    height = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) + MENU_MARGIN;
    if ((softkey = Softkey::GetInstance()) != null) {
        rect.SubBottom(softkey->GetBaseBound().GetHeight());
    }
    result.SetSize(rect.GetWidth() * 3 / 4, height * limit);
    result.Inflate(SFRTextMenu::GetMargin());
    result.SetOrigin(0, 0);
    result.Offset(SFXGrid(rect.GetSize() - result.GetSize()));
    return result;
}

SFCError psec::OpenStandardDialog(SFRDialogPtr dialog, SFRHandlerSPP procedure, VoidPtr reference)
{
    SoftkeyPtr softkey;
    SFCError error(SFERR_NO_ERROR);

    if ((error = dialog->static_catch()) == SFERR_NO_ERROR) {
        if ((error = dialog->RegisterHandler(SREVT_DIALOG, HANDLER_BEFORE, procedure, reference)) == SFERR_NO_ERROR) {
            Softkey::Initialize(dialog);
            if ((softkey = Softkey::GetInstance()) != null) {
                if ((error = softkey->Register(dialog, Softkey::SOFTKEY_3, LABEL_SOFTKEY3)) == SFERR_NO_ERROR) {
                    dialog->SetReference(softkey->Bind(dialog));
                }
            }
        }
    }
    if (error != SFERR_NO_ERROR) {
        ::delete dialog;
    }
    return error;
}

Void psec::CloseStandardDialog(SFRDialogPtr dialog)
{
    Softkey::Terminate(dialog);
    dialog->DialogHandler();
    return;
}

SFCError psec::OpenCustomDialog(SFRDialogPtr dialog, SFRHandlerSPP procedure, VoidPtr reference)
{
    SFCError error(SFERR_NO_ERROR);

    if ((error = dialog->static_catch()) == SFERR_NO_ERROR) {
        error = dialog->RegisterHandler(SREVT_DIALOG, HANDLER_BEFORE, procedure, reference);
    }
    if (error != SFERR_NO_ERROR) {
        ::delete dialog;
    }
    return error;
}

Void psec::CloseCustomDialog(SFRDialogPtr dialog)
{
    dialog->DialogHandler();
    return;
}

SFCError psec::OpenStandardMenu(SFRMenuPtr menu, SFRHandlerSPP procedure, VoidPtr reference)
{
    SoftkeyPtr softkey;
    SFCError error(SFERR_NO_ERROR);

    if ((error = menu->static_catch()) == SFERR_NO_ERROR) {
        if ((error = menu->RegisterHandler(SREVT_MENU, HANDLER_BEFORE, procedure, reference)) == SFERR_NO_ERROR) {
            if ((error = menu->RegisterHandler(static_cast<SFCEventEnum>(Softkey::USEREVT_SOFTKEY), HANDLER_AFTER, OnSoftkeySHP, menu)) == SFERR_NO_ERROR) {
                Softkey::Initialize(menu);
                if ((softkey = Softkey::GetInstance()) != null) {
                    if ((error = softkey->Register(menu, Softkey::SOFTKEY_2, LABEL_SOFTKEY2)) == SFERR_NO_ERROR) {
                        if ((error = softkey->Register(menu, Softkey::SOFTKEY_3, LABEL_SOFTKEY3)) == SFERR_NO_ERROR) {
                            menu->SetReference(softkey->Bind(menu));
                        }
                    }
                }
            }
        }
    }
    return error;
}

Void psec::CloseStandardMenu(SFRMenuPtr menu)
{
    Softkey::Terminate(menu);
    menu->MenuHandler();
    return;
}

HANDLER_IMPLEMENT_VOIDRENDER(psec, OnRender, graphics)
{
    static SFXBevelColor::AtomRecConst theme[] = {
        {
            {0x00, 0xCC, 0xCC, 0xCC},
            {0x00, 0x55, 0x55, 0x55},
            {0x00, 0x00, 0x00, 0x00}
        },
        {
            {0x00, 0x99, 0x99, 0x99},
            {0x00, 0x55, 0x55, 0x55},
            {0x00, 0x33, 0x33, 0x33}
        },
        {
            {0x00, 0x77, 0x77, 0x77},
            {0x00, 0x55, 0x55, 0x55},
            {0x00, 0x44, 0x44, 0x44}
        },
        {
            {0x00, 0x00, 0x00, 0x00},
            {0x00, 0x55, 0x55, 0x55},
            {0x00, 0xCC, 0xCC, 0xCC}
        },
        {
            {0x00, 0x33, 0x33, 0x33},
            {0x00, 0x55, 0x55, 0x55},
            {0x00, 0x55, 0x55, 0x55}
        }
    };
    SFXRectangle rect(GetContentWorld());
    SFRResponderPtr monitor;

    graphics->DrawBevelVertical(rect, theme[0]);
    rect.Deflate(1, 0);
    graphics->DrawBevelVertical(rect, theme[1]);
    rect.Deflate(1, 0);
    graphics->FillBevelVertical(rect, theme[2]);
    if ((monitor = Monitor::GetInstance()) != null) {
        rect = monitor->GetBaseBound();
        rect.Inflate(1, 1);
        graphics->DrawBevelRectangle(rect, theme[3]);
        rect.Inflate(1, 1);
        graphics->DrawBevelRectangle(rect, theme[4]);
    }
    return;
}

HANDLER_IMPLEMENT_VOIDSTART(psec, OnStart, environment)
{
    unused(environment);

    KeyboardPtr keyboard;
    SFRMessageDialogPtr dialog;
    SFXWideString text;
    SFBImageSmp image;
    SFCError error;

    error = _calculator->Initialize();
    if ((keyboard = MNKeyboard::GetInstance()) != null) {
        keyboard->Refresh();
        keyboard->Activate(null);
    }
    if (error != SFERR_NO_ERROR) {
        text = "It failed in reading the initialization file.";
        image = SFBShell::GetInstance()->LoadResImage(PSEC_RES_FILE, CANCEL);
        if ((dialog = ::new SFRMessageDialog(CalculateDialogPosition(SFRMessageDialog::GetMargin(), MESSAGEDIALOG_HEIGHT), text, "OK", image)) != null) {
            OpenStandardDialog(dialog, HANDLER_FUNCTION(FromMessage));
        }
    }
    return;
}

HANDLER_IMPLEMENT_VOIDSTOP(psec, OnStop, quitable)
{
    unused(quitable);

    _calculator->Terminate();
    return;
}

HANDLER_IMPLEMENT_VOIDEVENT(psec, FromMessage, event)
{
    switch (event.GetP16()) {
        case SRP16_OK:
            CloseStandardDialog(reinterpret_cast<SFRMessageDialogPtr>(event.GetP32()));
            break;
    }
    return;
}

Bool psec::OnSoftkeySHP(SFXEventConstRef event, VoidPtr reference)
{
    SFRMenuPtr menu;

    switch (event.GetP16()) {
        case Softkey::SOFTKEY_2:
            menu = static_cast<SFRMenuPtr>(reference);
            menu->Invoke(SFXEvent(SREVT_MENU, SRP16_CANCEL, reinterpret_cast<UInt32>(menu)));
            break;
    }
    return true;
}
